#
# Diameter Channel WSAdmin Jython Install Script
#
# This script sets up the Diameter Channel configuration
#
# Usage:
# wsadmin.sh -f <script_path> <cell> <node> <server_name> <host> <port> standalone|cluster [debug]
#     script_path:  path to this script
#     cell:  cell name (ie, myCell01)
#     node:  node name (ie, myNode01)
#     server_name:  server name (ie, server1)
#     host:  host name you wish to bind the Diameter Inbound TCP Channel to
#     port:  port for the Diameter Inbound TCP Channel
#     standalone|cluster:  pass the word 'standalone' as the sixt parameter if
#                          this is a standalone server, otherwise pass 'cluster'
#     debug:  optional, just pass the word 'debug' as the seventh parameter to
#             enable debug

import sys

# Takes in a list from a WSAdmin object and creates a Jython list out of it
def wsadminToList(inStr):
        outList=[]
        if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
                tmpList = inStr[1:-1].split(" ")
        else:
                tmpList = inStr.split("\n")  #splits for Windows or Linux
        for item in tmpList:
                item = item.rstrip();        #removes any Windows "\r"
                if (len(item)>0):
                        outList.append(item)
        return outList
#endDef

# Strips out the []'s and the specified string
def stripOutStringAndBrackets(string, value):
	retVal = string.replace('[' + value, '')
	retVal = retVal.replace(']', '')
	retVal = retVal.strip()
	return retVal
#endDef

# Returns if debug is enabled
def isDebugEnabled():
	if 7 == len(sys.argv) and 'debug' == sys.argv[6]:
		return 1
	else:
		return 0
	#endIf
#endDef

# Returns if this is a cluster
def isCluster():
	if 'cluster' == sys.argv[5]:
		return 1
	else:
		return 0
	#endIf
#endDef

# Prints the usage
def printUsage():
	print 'Usage:\n\
	wsadmin.sh -f <script name> <cell> <node> <server name> <host> <port> standalone|cluster [debug]\n'
#endDef

# Prints the Copyright
def printCopyright():
	print '***************************************************************\n\n\
	Licensed Materials - Property of IBM\n\
	5724-O05\n\
	(c) Copyright IBM Corp. 2006 All Rights Reserved.\n\
	* Trademark of International Business Machines\n'
	print '***************************************************************'
#endDef

printCopyright()

# Get the Server ID
if 6 > len(sys.argv):
	printUsage()
	sys.exit(2)
#endIf

cell = sys.argv[0]
node = sys.argv[1]
serverName = sys.argv[2]
host = sys.argv[3]
try:
	port = int(sys.argv[4])
except:
	print 'Invalid value for port "' + sys.argv[4] + '".  The port must be a valid port number for your system\n'
	printUsage()
	sys.exit(2)
#endExcept
cluster = isCluster()

# See if debug is enabled
debug = isDebugEnabled()
if debug:
	print 'debug is enabled!'
	print 'Parameters are:\n\
	cell:  ' + cell + '\n\
	node:  ' + node + '\n\
	serverName:  ' + serverName + '\n\
	host:  ' + host + '\n\
	port:  ' + str(port) + '\n\
	cluster:  ' + str(cluster)
	
#endIf

# Form the parameter string for the SSL config
scope = ''
parameters = ''
if (cluster):
	scope = ' (cell):' + cell + ' '
	parameters = '[-alias Diameter -scopeName' + scope + '-clientAuthentication true -trustStoreName CellDefaultTrustStore -trustStoreScopeName' + scope + '-keyStoreName CellDefaultKeyStore -keyStoreScopeName' + scope + ']'
else:
	scope = ' (cell):' + cell + ':(node):' + node + ' '
	parameters = '[-alias Diameter -scopeName' + scope + '-clientAuthentication true -trustStoreName NodeDefaultTrustStore -trustStoreScopeName' + scope + '-keyStoreName NodeDefaultKeyStore -keyStoreScopeName' + scope + ']'
#endIf

if debug:
	print 'Scope:  \n' + scope + '\n\
	Parameters:  \n' + parameters
	
	print 'Creating the SSL Config'
#endIf

AdminTask.createSSLConfig(parameters)

# Get the server id
serverID = AdminConfig.getid('/Server:' + serverName);
if debug:
	print 'ServerID is:  ' + serverID
#endIf

if '' == serverID:
	print 'Invalid Server Name "' + serverName + '"\n'
	printUsage()
	sys.exit(2)
#endIf

# Get the Server Name from AdminConfig
serverName = AdminConfig.show(serverID,'name')
if debug:
	print 'Server Name from AdminConfig:  ' + serverName
#endIf

# Strip out the 'name' and brackets
serverName = stripOutStringAndBrackets(serverName, 'name')
if debug:
	print 'Server Name stripped:   ' + serverName
#endIf

# Get the Node ID
nodeID = AdminConfig.getid('/Node:' + node)
if debug:
	print 'Node ID is:  ' + nodeID
#endIf

if '' == nodeID:
	print 'Invalid Node Name "' + node + '"\n'
	printUsage()
	sys.exit(2)
#endIf

# Get our server entries
entriesString = AdminConfig.list('ServerEntry', nodeID)
entries = wsadminToList(entriesString)

if debug:
	print 'Server Entries:  ' + str(entries)
#endIf

# Find the entry for our server
serverEntry = ''
for entry in entries:
	if debug:
		print 'Current entry is ' + entry
	#endIf
		
	serverEntryName = AdminConfig.show(entry, 'serverName')
	serverEntryName = stripOutStringAndBrackets(serverEntryName, 'serverName')
	
	if debug:
		print 'serverEntryName is. . .  ' + serverEntryName
        #endIf
        
        if (serverEntryName == serverName):
        	serverEntry = entry
        	if debug:
        		print 'Found our server entry:\n' + serverEntry
        	#endIf
        #endIf 
#endFor

# Create our endpoints
namedEndPoint = AdminConfig.create('NamedEndPoint', serverEntry, [['endPointName', 'DiameterNamedEndPoint']])
endPoint = AdminConfig.create('EndPoint', namedEndPoint, [['host', host], ['port', port]])
if debug:
	print 'Named Endpoint is ' + namedEndPoint
	print 'End Point is ' + endPoint
#endIf

# Create the ThreadPool
TPM = AdminConfig.list('ThreadPoolManager', serverID)
diameterThreadPool = AdminConfig.create('ThreadPool', TPM, [['name', 'DiameterThreadPool'], ['description', 'Diameter Channel Thread Pool'], ['minimumSize', 4], ['maximumSize', 50], ['inactivityTimeout', -1]])
if debug:
	print 'TPM is ' + TPM
	print 'Diameter Thread Pool is ' + diameterThreadPool
#endIf

# Create the Channels and the Chains
TCS = AdminConfig.list('TransportChannelService', serverID)
tcpin = AdminConfig.create('TCPInboundChannel', TCS, [['name', 'DiameterTCPInboundChannel'],['endPointName', 'DiameterNamedEndPoint'],['threadPool', diameterThreadPool]])
diamin = AdminConfig.create('GenericInboundChannel', TCS, [['name', 'DiameterGenericInboundChannel'],['jarFile', 'com.ibm.ws.diameter_6.1.0.jar'],['discriminationWeight', 10],['typeID', 'DiameterInboundChannel']])
inchain = AdminConfig.create('Chain', TCS, [['name', 'DiameterChain'],['enable', 'true'],['transportChannels', [tcpin, diamin]]] )
sslin = AdminConfig.create('SSLInboundChannel', TCS, [['name', 'DiameterSSLInboundChannel'],['discriminationWeight', 1],['sslConfigAlias', 'Diameter']])
diamsecurein = AdminConfig.create('GenericInboundChannel', TCS, [['name', 'SecureDiameterGenericInboundChannel'],['jarFile', 'com.ibm.ws.diameter_6.1.0.jar'],['discriminationWeight', 1],['typeID', 'DiameterInboundChannel']])
insecurechain = AdminConfig.create('Chain', TCS, [['name', 'SecureDiameterChain'],['enable', 'true'],['transportChannels', [tcpin, sslin, diamsecurein]]] )
if debug:
	print 'TCS is ' + TCS
	print 'tcpin is ' + tcpin
	print 'sslin is ' + sslin
	print 'diamin is ' + diamin
	print 'diamsecurein is ' + diamsecurein
	print 'inchain is ' + inchain
	print 'insecurechain is ' + insecurechain
#endIf

# Create the workmanager
WMP = AdminConfig.list('WorkManagerProvider', serverID)
workManager = AdminConfig.create('WorkManagerInfo', WMP, [['name', 'DiameterWorkManager'],['jndiName','wm/diameter'],['minThreads',10],['maxThreads',10],['threadPriority',5],['numAlarmThreads',2],['isGrowable','true'],['workReqQFullAction',0],['serviceNames','security;UserWorkArea;com.ibm.ws.i18n']])
if debug:
	print 'WMP is ' + WMP
	print 'WorkManager is ' + workManager
#endIf
	
# Save it! 
AdminConfig.save()
if debug:
	print 'Config has been saved'
#endIf

print 'Diameter Channels successfully configured for server ' + serverName + ' on node ' + node
